/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.jni.CPointer;

/**
 * This value Class represents the VCI Device Capabilities. The VCI Device
 * Capabilities can be retrieved by calling {@link IVciDevice#GetDeviceCaps()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class VciDeviceCapabilities
{
  /** maximum possible bus sockets */
  public static final short VCI_MAX_BUSCTRL = 32;

  // VCI Bus Types
  /** reserved */
  public static final short VCI_BUS_RES = 0; 
  /** CAN Bus Type*/
  public static final short VCI_BUS_CAN = 1; 
  /** LIN Bus Type*/
  public static final short VCI_BUS_LIN = 2;
  /** FlexRay Bus Type*/
  public static final short VCI_BUS_FLX = 3;

  /** Unknown controller Type */
  public static final short VCI_CTRL_UNKNOWN    = 0;

  // CAN Controller Types
  /**
   * Unknown CAN controller Type
   * @see de.ixxat.vci3.bal.can.CanCapabilities#CAN_CTRL_UNKNOWN
   */
  public static final short CAN_CTRL_UNKNOWN    = 0;

  // LIN Controller Types
  /** Unknown LIN controller Type */
  public static final short LIN_CTRL_UNKNOWN    = 0;

  // FlexRay Controller Types
  /** Unknown FlexRay controller Type */
  public static final short FLX_CTRL_UNKNOWN    = 0;
  
  /** number of supported bus controllers */
  public short m_wBusCtrlCount          = 0;
  /** array of supported bus controllers */
  private short m_awBusCtrlType[]       = new short[VCI_MAX_BUSCTRL];

  /** VCI BUS Type */
  public short    m_awBusType[]         = new short[VCI_MAX_BUSCTRL];
  /** VCI Controller Type*/
  public short    m_awControllerType[]  = new short[VCI_MAX_BUSCTRL];

  /**
   * Creates a new VCI Device Capability object.
   * @throws Throwable
   */
  protected VciDeviceCapabilities()
  {
  }

  /**
   * Creates a new VCI Device Capability object.
   * 
   * @param pVciDeviceCaps
   *          CPointer to native memory which contains the VCIDEVICECAPS
   *          structure.
   * @throws Throwable
   */
  protected VciDeviceCapabilities(CPointer pVciDeviceCaps) throws Throwable
  {
    fromCPointer(pVciDeviceCaps);
  }

  /**
   * Updates a existing VCI Device Capability object.
   * 
   * @param pVciDeviceCaps
   *          CPointer to native memory which contains the VCIDEVICECAPS
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pVciDeviceCaps) throws Throwable
  {
    // Update VCI Device Capabilities
    m_wBusCtrlCount   = pVciDeviceCaps.getShort(0);
    
    // Iterate trough all possible Sockets
    for(short wIndex =0; wIndex < VCI_MAX_BUSCTRL; wIndex++)
    {
      if(wIndex < m_wBusCtrlCount)
        m_awBusCtrlType[wIndex] = pVciDeviceCaps.getShort(2+(wIndex*2));
      else
        m_awBusCtrlType[wIndex] = VCI_BUS_RES;

      m_awBusType[wIndex]         = (short)((m_awBusCtrlType[wIndex] >> 8) & 0x00FF);
      m_awControllerType[wIndex]  = (short)((m_awBusCtrlType[wIndex] >> 0) & 0x00FF);
    }
  }

  /**
   * Returns the VCI Device Capabilities as string.
   * @return The VCI Device Capabilities as string.
   */
  public String toString()
  {
    String strVciDeviceCaps = "";
    strVciDeviceCaps += "Controller count: " + m_wBusCtrlCount + " (Bus, Controller) -> {";
    
    // Iterate trough all available Sockets
    for(short wIndex =0; wIndex < m_wBusCtrlCount; wIndex++)
    {
      strVciDeviceCaps += "(" + m_awBusType[wIndex] + ", ";
      strVciDeviceCaps += m_awControllerType[wIndex]+ ")";
      if((wIndex+1) < m_wBusCtrlCount)
        strVciDeviceCaps += ", ";
      else
        strVciDeviceCaps += "}";
    }

    return strVciDeviceCaps;
  }

}
