/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;

/**
 * This class implements the interface {@link IVciDeviceManager} which gives
 * access to the list of installed VCI devices or opens a VCI device.
 * 
 * @see IVciDeviceManager
 * @author Michael Ummenhofer
 * 
 */
class VciDeviceManager extends CInterface implements IVciDeviceManager
{
  // Interface Method Index
  private static final int EnumDevices    = 3;
  private static final int OpenDevice     = 4;

  // Interface functions
  private CFunction m_fpEnumDevices       = null;
  private CFunction m_fpOpenDevice        = null;

  // Interface Pointer
  private CPointer  m_IVciDeviceManager   = null; //Interface Pointer itself (pThis)
  
  /**
   * constructor
   * 
   * @param pIVciDeviceManager
   * @throws Throwable
   */
  protected VciDeviceManager(CMalloc pIVciDeviceManager) throws Throwable
  {
    super(pIVciDeviceManager);

    try
    {
      // Retrieve Interface Pointer Address
      m_IVciDeviceManager = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpEnumDevices     = new CFunction(m_IVciDeviceManager, EnumDevices,    "JNI");
      m_fpOpenDevice      = new CFunction(m_IVciDeviceManager, OpenDevice,     "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_IVciDeviceManager = null;
      m_fpEnumDevices     = null;
      m_fpOpenDevice      = null;
 
      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_IVciDeviceManager = null;
      m_fpEnumDevices     = null;
      m_fpOpenDevice      = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }

  /**
   * @see de.ixxat.vci3.IVciDeviceManager#SelectDeviceDialog()
   * @throws Throwable
   */
  public long SelectDeviceDialog() throws Throwable
  {
    long    lVciId  = -1;
    int     hResult = 0;

    CMalloc pVciId = new CMalloc(8);
    try 
    {
        // DLL Function definitions
        CFunction fpVciSelectDeviceDlg = null;
        CFunction fpInitCommonControls = null;
         
        // Ensure windows common controls are loaded
        System.loadLibrary("comctl32");
        fpInitCommonControls = new CFunction("comctl32", "InitCommonControls", "C");
        fpInitCommonControls.callVoid(new Object[]{});
        
        fpVciSelectDeviceDlg = new CFunction("Vcinpl", "vciSelectDeviceDlg", "JNI");
      pVciId.setLong(0, 0);
      hResult = fpVciSelectDeviceDlg.callInt(new Object[]{ new Integer(0), pVciId});
      if(hResult != 0)
      {
        throw new VciException("VciSelectDeviceDlg failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        lVciId = pVciId.getLong(0);
      }
    } 
    finally 
    {
      pVciId.free();
    }
    
    return lVciId;
  }
  
  /**
   * @see de.ixxat.vci3.IVciDeviceManager#EnumDevices()
   * @throws Throwable
   */
  public IVciEnumDevice EnumDevices() throws Throwable
  {
    VciEnumDevice oVciEnumDevice = null;
    int           hResult        = 0;

    // Create VciEnumDevice Interface Memory
    CMalloc pIVciEnumDevice = new CMalloc(8);
    try
    {
      pIVciEnumDevice.setLong(0, 0);

      // Retrieve IVciEnumDevice Interface
      hResult = m_fpEnumDevices.callInt(new Object[]{ m_IVciDeviceManager, pIVciEnumDevice});
      if(hResult != 0)
      {
        throw new VciException("IVciDeviceManager->EnumDevices failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new VciEnumDevice Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the VciEnumDevice Object
        oVciEnumDevice = new VciEnumDevice(pIVciEnumDevice);
      }
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pIVciEnumDevice.free();
      throw oException;
    }
    finally
    {
    }
    
    return oVciEnumDevice;
  }

  /**
   * @see de.ixxat.vci3.IVciDeviceManager#OpenDevice(long)
   * @throws Throwable
   */
  public IVciDevice OpenDevice(long lVciId) throws Throwable
  {
    VciDevice oVciDevice = null;
    int       hResult    = 0;

    // Create VciDevice Interface Memory
    CMalloc pIVciDevice = new CMalloc(8);
    CMalloc pVciId      = new CMalloc(8);
    try
    {
      pIVciDevice.setLong(0, 0);
      pVciId.setLong(0, lVciId);

      // Retrieve IVciDeviceManger Interface
      hResult = m_fpOpenDevice.callInt(new Object[]{ m_IVciDeviceManager, pVciId, pIVciDevice});
      if(hResult != 0)
      {
        throw new VciException("IVciDeviceManager->OpenDevice failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new VciDevice Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the VciDevice Object
        oVciDevice = new VciDevice(pIVciDevice);
      }
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pIVciDevice.free();
      throw oException;
    }
    finally
    {
      // Free VCI ID Memory
      pVciId.free();
    }
    
    return oVciDevice;
  }
}
