/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;

/**
 * This class represents the entry point for working with the VCI. Use
 * {@link VciServer#GetDeviceManager} to get access to the installed VCI
 * devices. When no longer needed the VCI Server object has to be
 * disposed using the {@link IDisposable} interface.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class VciServer implements IDisposable
{
  // DLL exported functions
  private static final CFunction m_fpVciInitialize;
  private static final CFunction m_fpVciGetVersion;
  private static final CFunction m_fpVciGetDeviceManager;
  
  // Initialization of static members
  static
  {
    m_fpVciInitialize       = new CFunction("vciapi", "VciInitialize",        "C");
    m_fpVciGetVersion       = new CFunction("vciapi", "VciGetVersion",        "JNI");
    m_fpVciGetDeviceManager = new CFunction("vciapi", "VciGetDeviceManager",  "JNI");
  }
  
  /**
   * The VCI Version Object.
   */
  private VciVersion m_oVciVersion = null;
     
  /**
   * Creates and Initializes a new VCI Server Object which serves as entry point
   * for working with the VCI.
   * 
   * @throws Throwable
   */
  public VciServer() throws Throwable
  {
    // Initialize the newly created object.
    Initialize();
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
    }
    finally
    {
    }
  }
  
  
  /**
   * Initializes the VCI V3 API
   */
  protected void Initialize() throws Throwable
  {
    int hResult = 0;
    hResult = m_fpVciInitialize.callInt(new Object[]{ null });
    
    // Succeeded?
    if(hResult == 0)
    {
      // Get VCI Version
      CMalloc pVciVersionInfo = new CMalloc(6*4);
      try 
      {
        byte abData[] = new byte[6*4]; // VCIVERSIONINFO
        pVciVersionInfo.copyIn(0, abData, 0, 6*4);
        hResult = m_fpVciGetVersion.callInt(new Object[]{ pVciVersionInfo });
        if(hResult == 0)
        {
          m_oVciVersion = new VciVersion(pVciVersionInfo);
        }
        else
          throw new VciException("VciGetVersion failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      finally 
      {
        pVciVersionInfo.free();
      }
    }//end if
    else
    {
      throw new VciException("VciInitialize failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
    }
  }

  /**
   * Gets the reference to a new VCI device manager instance. When no longer
   * needed the VCI device manager object has to be disposed using the
   * {@link IDisposable} interface.
   * 
   * @return {@link IVciDeviceManager} - A reference to the VCI device manager.
   * @throws Throwable
   */
  public IVciDeviceManager GetDeviceManager() throws Throwable
  {
    VciDeviceManager oDeviceManager = null;
    int              hResult        = 0;

    // Create Device Manager Interface Memory
    CMalloc pIVciDeviceManager = new CMalloc(8);
    try
    {
      pIVciDeviceManager.setLong(0, 0);
  
      // Retrieve IVciDeviceManger Interface
      hResult = m_fpVciGetDeviceManager.callInt(new Object[]{ pIVciDeviceManager });
      if(hResult != 0)
      {
        throw new VciException("VciGetDeviceManager failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new Device Manager Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the DeviceManager Object
        oDeviceManager = new VciDeviceManager(pIVciDeviceManager);
      }
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pIVciDeviceManager.free();
      throw oException;
    }
    
    return oDeviceManager;
  }
  
  /**
   * Returns the VCI Version.
   * @return {@link VciVersion} - The VCI Version Object
   */
  public VciVersion GetVersion()
  {
    
    return m_oVciVersion;
  }

  
 

}
