/**
 * 
 */
package de.ixxat.vci3.bal;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the base functionality of a FIFO.
 *  
 * @author Michael Ummenhofer
 *
 */
class Fifo extends CInterface
{
  private static final int Lock           = 3;
  private static final int Unlock         = 4;
  private static final int AssignEvent    = 5;
  private static final int SetThreshold   = 6;
  private static final int GetThreshold   = 7;
  private static final int GetCapacity    = 8;
  private static final int GetEntrySize   = 9;
  private static final int GetFillCount   = 10;
  private static final int GetFreeCount   = 11;

  // Interface functions
  private CFunction m_fpLock            = null;
  private CFunction m_fpUnlock          = null;
  private CFunction m_fpAssignEvent     = null;
  private CFunction m_fpSetThreshold    = null;
  private CFunction m_fpGetThreshold    = null;
  private CFunction m_fpGetCapacity     = null;
  private CFunction m_fpGetEntrySize    = null;
  private CFunction m_fpGetFillCount    = null;
  private CFunction m_fpGetFreeCount    = null;
  
  // FIFO C-Event Handle
  private CEvent    m_oFifoEvent        = new CEvent();
  private boolean   m_fWaitCanceled     = false;
  private boolean   m_fWaitEvent        = false;

  // Instance name
  protected String  m_strInstanceName   = "IFifo";
  
  // Interface Pointer
  protected CPointer  m_IFifo   = null; //Interface Pointer itself (pThis)

  // Memory for FIFO Entry access
  protected CMalloc m_ppvData = null;
  protected CMalloc m_pwCount = null;

  /**
   * Constructor of a new FIFO Object
   * 
   * @param pIFifo
   *          FIFO Interface
   * @param strInstanceName
   *          Instance Name (e.g. "FifoReader")
   * @param dwThreshold
   *          Threshold for Event handling. For RX FIFOs the event is signaled
   *          for every new received message which exceeds the given threshold.
   *          For TX FIFOs the event is signaled for every removed message if
   *          the free entries exceeds the threshold. If the given threshold is
   *          zero the event handling is not available.
   * @throws Throwable
   */
  protected Fifo(CMalloc pIFifo, String strInstanceName, int dwThreshold) throws Throwable
  {
    super(pIFifo);
    
    try
    {
      int hResult = 0;
      
      // Store Instance Name
      m_strInstanceName = strInstanceName;
      
      // Retrieve Interface Pointer Address
      m_IFifo = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpLock            = new CFunction(m_IFifo, Lock,           "JNI");
      m_fpUnlock          = new CFunction(m_IFifo, Unlock,         "JNI");
      m_fpAssignEvent     = new CFunction(m_IFifo, AssignEvent ,   "JNI");
      m_fpSetThreshold    = new CFunction(m_IFifo, SetThreshold,   "JNI");
      m_fpGetThreshold    = new CFunction(m_IFifo, GetThreshold,   "JNI");
      m_fpGetCapacity     = new CFunction(m_IFifo, GetCapacity ,   "JNI");
      m_fpGetEntrySize    = new CFunction(m_IFifo, GetEntrySize,   "JNI");
      m_fpGetFillCount    = new CFunction(m_IFifo, GetFillCount,   "JNI");
      m_fpGetFreeCount    = new CFunction(m_IFifo, GetFreeCount,   "JNI");

      m_ppvData = new CMalloc(8);
      m_pwCount = new CMalloc(2);

      // Threshold given
      if(dwThreshold > 0)
      {
        // Assign Event Handle
        hResult = m_fpAssignEvent.callInt(new Object[]{ m_IFifo, m_oFifoEvent.GetHandle() });
        if(hResult != 0)
        {
          throw new VciException(m_strInstanceName + "->AssignEvent failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
        }
  
        // Set Threshold to (dwThreshold) for Event signaling
        hResult = m_fpSetThreshold.callInt(new Object[]{ m_IFifo, new Integer(dwThreshold) });
        if(hResult != 0)
        {
          throw new VciException(m_strInstanceName + "->SetThreshold failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
        }
      }
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_IFifo             = null;
      m_fpLock            = null;
      m_fpUnlock          = null;
      m_fpAssignEvent     = null;
      m_fpSetThreshold    = null;
      m_fpGetThreshold    = null;
      m_fpGetCapacity     = null;
      m_fpGetEntrySize    = null;
      m_fpGetFillCount    = null;
      m_fpGetFreeCount    = null;
      
      // Close FIFO C-Event
      if(m_oFifoEvent != null)
      {
        m_oFifoEvent.CloseHandle();
        m_oFifoEvent = null;
      }

      // Free Memory
      if(m_ppvData != null)
      {
        m_ppvData.free();
        m_ppvData = null;
      }
      if(m_pwCount != null)
      {
        m_pwCount.free();
        m_pwCount = null;
      }

      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_IFifo             = null;
      m_fpLock            = null;
      m_fpUnlock          = null;
      m_fpAssignEvent     = null;
      m_fpSetThreshold    = null;
      m_fpGetThreshold    = null;
      m_fpGetCapacity     = null;
      m_fpGetEntrySize    = null;
      m_fpGetFillCount    = null;
      m_fpGetFreeCount    = null;
      
      // Close FIFO C-Event
      if(m_oFifoEvent != null)
      {
        m_oFifoEvent.CloseHandle();
        m_oFifoEvent = null;
      }
  
      // Free Memory
      if(m_ppvData != null)
      {
        m_ppvData.free();
        m_ppvData = null;
      }
      if(m_pwCount != null)
      {
        m_pwCount.free();
        m_pwCount = null;
      }
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
 
  /**
   * Locks the FIFO access. 
   * 
   * @throws Throwable
   */
  protected void Lock() throws Throwable
  {
    int hResult     = 0;

    try
    {
      hResult = m_fpLock.callInt(new Object[]{ m_IFifo });
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->Lock failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
  
  /**
   * Unlocks the FIFO access.
   *  
   * @throws Throwable
   */
  protected void Unlock() throws Throwable
  {
    int hResult     = 0;

    try
    {
      hResult = m_fpUnlock.callInt(new Object[]{ m_IFifo });
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->Unlock failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
 
  /**
   * Gets the capacity of the FIFO in number entries.
   * 
   * @return Capacity
   * @throws Throwable
   */
  public int GetCapacity() throws Throwable
  {
    int hResult     = 0;
    int dwCapacity  = 0;
    
    // Create Memory for return value
    CMalloc pwCapacity = new CMalloc(2);
    
    try
    {
      hResult = m_fpGetCapacity.callInt(new Object[]{ m_IFifo, pwCapacity});
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->GetCapacity failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        dwCapacity = pwCapacity.getShort(0);
      }
    }
    finally
    {
      // Free Memory
      pwCapacity.free();
    }
   
    return dwCapacity;
  }
  
  /**
   * Gets the entry size of the FIFO in number bytes.
   * 
   * @return EntrySize
   * @throws Throwable
   */
  protected int GetEntrySize() throws Throwable
  {
    int hResult     = 0;
    int dwEntrySize = 0;
    
    // Create Memory for return value
    CMalloc pwEntrySize = new CMalloc(2);
    
    try
    {
      hResult = m_fpGetEntrySize.callInt(new Object[]{ m_IFifo, pwEntrySize});
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->GetEntrySize failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        dwEntrySize = pwEntrySize.getShort(0);
      }
    }
    finally
    {
      // Free Memory
      pwEntrySize.free();
    }
   
    return dwEntrySize;
  }
  
  /**
   * Gets the number of currently occupied entries within the FIFO.
   * 
   * @return FillCount
   * @throws Throwable
   */
  public int GetFillCount() throws Throwable
  {
    int hResult     = 0;
    int dwFillCount = 0;
    
    // Create Memory for return value
    CMalloc pwFillCount = new CMalloc(2);
    
    try
    {
      hResult = m_fpGetFillCount.callInt(new Object[]{ m_IFifo, pwFillCount});
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->GetFillCount failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        dwFillCount = pwFillCount.getShort(0);
      }
    }
    finally
    {
      // Free Memory
      pwFillCount.free();
    }
   
    return dwFillCount;
  }
  
  /**
   * Gets the number of currently free entries within the FIFO.
   * 
   * @return FreeCount
   * @throws Throwable
   */
  public int GetFreeCount() throws Throwable
  {
    int hResult     = 0;
    int dwFreeCount = 0;
    
    // Create Memory for return value
    CMalloc pwFreeCount = new CMalloc(2);
    
    try
    {
      hResult = m_fpGetFreeCount.callInt(new Object[]{ m_IFifo, pwFreeCount});
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->GetFreeCount failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        dwFreeCount = pwFreeCount.getShort(0);
      }
    }
    finally
    {
      // Free Memory
      pwFreeCount.free();
    }
   
    return dwFreeCount;
  }
  
  /**
   * The WaitFor function returns when the FIFO Event object is in the
   * signaled state or the time-out interval elapses. If the time-out value has
   * elapsed or the event has been destroyed this function throws an exception.
   * 
   * @param dwMilliseconds
   *          Time-out interval, in milliseconds. The function returns if the
   *          interval elapses, even if the event is non- signaled. If
   *          dwMilliseconds is {@link CEvent#TEST}, the function tests the
   *          event's state and returns immediately. If dwMilliseconds is
   *          {@link CEvent#INFINITE}, the function's time-out interval never
   *          elapses.
   * @throws Throwable
   */
  public void WaitFor(int dwMilliseconds) throws Throwable
  {
    // reset Wait Canceled Flag
    synchronized (this)
    {
      m_fWaitCanceled = false;
      m_fWaitEvent    = true;
    }//end sync

    // Wait for Event
    m_oFifoEvent.WaitFor(dwMilliseconds);

    // check if wait was abandoned by user
    synchronized (this)
    {
      m_fWaitEvent = false;

      if(m_fWaitCanceled)
      {
        m_fWaitCanceled = false;
        throw new VciException("WaitFor abandoned manually", (int)0x80004004 /*E_ABORT*/);
      }
    }//end sync
  }
  
  /**
   * The CancelWait function unblocks a thread waiting in
   * {@link Fifo#WaitFor(int)}. If any thread was waiting it will return with
   * an exception.
   * 
   * @throws Throwable
   */
  public void CancelWait() throws Throwable
  {
    synchronized (this)
    {
      // If a thread is waiting
      if(m_fWaitEvent)
      {
        // Signal abort
        m_fWaitCanceled = true;

        // Simulate Pulse event
        m_oFifoEvent.SetEvent();
        m_oFifoEvent.ResetEvent();
      }
    }//end sync
  }
  
}
