/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.vci3.IDisposable;

/**
 * This interface represents a LIN message reader. When no longer needed the LIN
 * message reader object has to be disposed using the {@link IDisposable}
 * interface. It's used to read received LIN messages from a LIN monitor
 * channel (see {@link ILinMonitor}). A LIN message reader object can be
 * obtained via the method {@link ILinMonitor#GetMessageReader()}.
 * 
 * @author Michael Ummenhofer
 */
public interface ILinMessageReader extends IDisposable
{
  /**
   * Predefined timeout interval for {@link #WaitFor(int)}.
   * 
   * @see #WaitFor(int)
   */
  public static final int INFINITE = -1;

  /**
   * Gets the capacity of the receive FIFO in number of LIN messages.
   * 
   * @return Capacity
   * @throws Throwable
   */
  public int GetCapacity() throws Throwable;

  /**
   * Gets the number of currently unread LIN messages within the receive FIFO.
   * 
   * @return FillCount
   * @throws Throwable
   */
  public int GetFillCount() throws Throwable;

  /**
   * Gets the number of currently free LIN messages within the receive FIFO.
   * 
   * @return FreeCount
   * @throws Throwable
   */
  public int GetFreeCount() throws Throwable;

  /**
   * The WaitFor function returns when a new LIN message was received or the
   * time-out interval elapses. If the time-out value has elapsed or the waiting
   * was canceled by a call to {@link #CancelWait()} the function returns with
   * an exception.
   * 
   * @param dwMilliseconds
   *          Time-out interval, in milliseconds. The function returns if the
   *          interval elapses, even if no message was received. If
   *          dwMilliseconds is {@link #INFINITE}, the function's time-out
   *          interval never elapses.
   * @throws Throwable
   */
  public void WaitFor(int dwMilliseconds) throws Throwable;

  /**
   * Unblocks a thread waiting in {@link #WaitFor(int)}. If any thread was
   * waiting, the waiting thread will return with an exception.
   * 
   * @see #WaitFor(int)
   * 
   * @throws Throwable
   */
  public void CancelWait() throws Throwable;

  /**
   * This method reads a single LIN message from the front of the receive FIFO
   * and removes the message from the FIFO.
   * <p>
   * <strong>Remarks:</strong><blockquote>This method creates for every call a
   * new LinMessage object which has to be garbage collected by the runtime
   * environment later. Use {@link #ReadMessage(LinMessage)} instead.</blockquote>
   * 
   * @return LinMessage The LinMessage received.
   * @throws Throwable
   *           If the reception fails (e.g. the FIFO is empty) this function
   *           throws an exception.
   */
  public LinMessage ReadMessage() throws Throwable;

  /**
   * This method reads a single LIN message from the front of the receive FIFO
   * and removes the message from the FIFO.
   * <p>
   * <strong>Remarks:</strong><blockquote>This method is faster than
   * {@link #ReadMessage()}, but the user must provide the LinMessage Object
   * which members are updated for every new LinMessage received.</blockquote>
   * 
   * @param oLinMsg
   *          Reference to a LinMessage where the method stores the read
   *          message.
   * @return LinMessage The LinMessage received.
   * @throws Throwable
   *           If the reception fails (e.g. the FIFO is empty) this function
   *           throws an exception.
   */
  public LinMessage ReadMessage(LinMessage oLinMsg) throws Throwable;

}
