/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;
import de.ixxat.vci3.bal.*;

/**
 * This class implements the interface {@link ILinSocket} which represents a LIN
 * controller properties and capabilities container.
 * 
 * @see ILinSocket
 * @author Michael Ummenhofer
 * 
 */
public class LinSocket extends CInterface implements ILinSocket
{
  private static final int GetSocketInfo  = 3;
  private static final int GetCapabilities= 4;
  private static final int GetLineStatus  = 5;
  private static final int CreateMonitor  = 6;

  // Interface functions
  private CFunction m_fpGetSocketInfo   = null;
  private CFunction m_fpGetCapabilities = null;
  private CFunction m_fpGetLineStatus   = null;
  private CFunction m_fpCreateMonitor   = null;

  // Interface Pointer
  private CPointer  m_ILinSocket        = null; //Interface Pointer itself (pThis)
  
  /** BAL Socket Info */
  private BalSocketInfo m_oBalSocketInfo      = new BalSocketInfo();
  
  /** LIN Capabilities */
  private LinCapabilities m_oLinCapabilities  = new LinCapabilities();

  /** LIN Line Status */
  private LinLineStatus m_oLinLineStatus      = new LinLineStatus();

  //TODO: Is there an other way to create this object -> "protected"
  /**
   * Constructor
   * 
   * @param pILinSocket
   * @throws Throwable
   */
  public LinSocket(CMalloc pILinSocket) throws Throwable
  {
    super(pILinSocket);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_ILinSocket = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpGetSocketInfo   = new CFunction(m_ILinSocket, GetSocketInfo,  "JNI");
      m_fpGetCapabilities = new CFunction(m_ILinSocket, GetCapabilities,"JNI");
      m_fpGetLineStatus   = new CFunction(m_ILinSocket, GetLineStatus,  "JNI");
      m_fpCreateMonitor   = new CFunction(m_ILinSocket, CreateMonitor,  "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_ILinSocket        = null;
      m_fpGetSocketInfo   = null;
      m_fpGetCapabilities = null;
      m_fpGetLineStatus   = null;
      m_fpCreateMonitor   = null;

      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ILinSocket        = null;
      m_fpGetSocketInfo   = null;
      m_fpGetCapabilities = null;
      m_fpGetLineStatus   = null;
      m_fpCreateMonitor   = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see ILinSocket#GetCapabilities()
   * @throws Throwable
   */
  public LinCapabilities GetCapabilities() throws Throwable
  {
    int hResult = 0;

    // Create LINCAPABILITIES Memory
    CMalloc pLinCapabilities = new CMalloc(32);
    try
    {
      hResult = m_fpGetCapabilities.callInt(new Object[] { m_ILinSocket, pLinCapabilities });
      if (hResult != 0)
      {
        throw new VciException("ILinSocket->GetCapabilities failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oLinCapabilities.fromCPointer(pLinCapabilities);
      }
    } 
    finally
    {
      pLinCapabilities.free();
    }
    
    return m_oLinCapabilities;
  }
  
  /**
   * @see ILinSocket#GetSocketInfo()
   * @throws Throwable
   */
  public BalSocketInfo GetSocketInfo() throws Throwable
  {
    int hResult = 0;

    // Create BALSOCKETINFO Memory
    CMalloc pBalSocketInfo = new CMalloc(12);
    try
    {
      hResult = m_fpGetSocketInfo.callInt(new Object[] { m_ILinSocket, pBalSocketInfo });
      if (hResult != 0)
      {
        throw new VciException("ILinSocket->GetSocketInfo failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oBalSocketInfo.fromCPointer(pBalSocketInfo);
      }
    } 
    finally
    {
      pBalSocketInfo.free();
    }
    
    return m_oBalSocketInfo;
  }

  /**
   * @see ILinSocket#GetLineStatus()
   * @throws Throwable
   */
  public LinLineStatus GetLineStatus() throws Throwable
  {
    int hResult = 0;

    // Create LINLINESTATUS Memory
    CMalloc pLinLineStatus = new CMalloc(8);
    try
    {
      hResult = m_fpGetLineStatus.callInt(new Object[] { m_ILinSocket, pLinLineStatus });
      if (hResult != 0)
      {
        throw new VciException("ILinSocket->GetLineStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oLinLineStatus.fromCPointer(pLinLineStatus);
      }
    } 
    finally
    {
      pLinLineStatus.free();
    }
    
    return m_oLinLineStatus;
  }

  /**
   * @see ILinSocket#CreateMonitor(boolean)
   * @throws Throwable
   */
  public ILinMonitor CreateMonitor(boolean fExclusive) throws Throwable
  {
    LinMonitor oLinMonitor = null;
    int        hResult     = 0;

    // Create LinMonitor Interface Memory
    CMalloc pILinMonitor = new CMalloc(8);
    try
    {
      pILinMonitor.setLong(0, 0);
      int nExclusive = 0;
      
      // Exclusive Channel?
      if(fExclusive)
        nExclusive = 1;

      // Retrieve ILinMonitor Interface
      hResult = m_fpCreateMonitor.callInt(new Object[]{ m_ILinSocket, new Integer(nExclusive), pILinMonitor});
      if(hResult != 0)
      {
        throw new VciException("ILinSocket->CreateMonitor failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new LinMonitor Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the LinMonitor Object
        oLinMonitor = new LinMonitor(pILinMonitor);
      }
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pILinMonitor.free();
      throw oException;
    }
    finally
    {
    }

    return oLinMonitor;
  }

}
